--[[
LibRule - library for item select rules

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local MAJOR = 'LibRule-1.0'
local MINOR = 2
local lib = LibStub:NewLibrary( MAJOR, MINOR)
if not lib then return end -- No Upgrade needed.

local qualities = {}
local cache = {}

--[[---------------------------------------------------------------------------------
  local functions
------------------------------------------------------------------------------------]]
local function GetCache( compiled)
	return cache[compiled]
end

local function SetCache( compiled, fkt)
	cache[compiled] = fkt
end

local function GetQuality( rarity)
	local q = qualities[rarity]
	if not q then
		q = getglobal( 'ITEM_QUALITY'..rarity..'_DESC')
		qualities[rarity] = q
	end
	return q
end

local function GetItemID( link)
	local id = link and link:match( '|Hitem:(%d+)') -- '|c%s|Hitem:%i:0:0:0:0:0:0:0|h[%s]|h|r'
	return tonumber( id)
end

local function GetItemValue( info, link)
	local iName, iLink, iRarity, iLevel, iMinLevel, iType, iSubType, iStackCount, iEquipLoc, iTexture, iPrice = GetItemInfo( link)
	if info == 'name' then
		return iName
	end
	if info == 'id' then
		return GetItemID( iLink) or 0
	end
	if info == 'quality' then
		return GetQuality( iRarity or 0)
	end
	if info == 'level' then
		return tonumber( iLevel) or 0
	end
	if info == 'uselevel' then
		return tonumber( iMinLevel) or 0
	end
	if info == 'type' then
		return iType
	end
	if info == 'subtype' then
		return iSubType
	end
	if info == 'count' then
		return tonumber( iStackCount) or 1
	end
	if info == 'equip' then
		return iEquipLoc and _G[iEquipLoc]
	end
	if info == 'price' then
		return tonumber( iPrice) or 0
	end
	return ''
end

--[[---------------------------------------------------------------------------------
  environment functions
------------------------------------------------------------------------------------]]
local Environment = {
	['name'] = function( link, ...)
		local iName = GetItemValue( 'name', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			if iName == value then
				return true
			end
		end
		return false
	end,
	
	['id'] = function( link, ...)
		local iID = GetItemValue( 'id', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			local id = tonumber( value) or 0
			if iID == id then
				return true
			end
		end
		return false
	end,
	
	['quality'] = function( link, ...)
		local iQuality = GetItemValue( 'quality', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			if iQuality == value then
				return true
			end
		end
		return false
	end,
	
	['level'] = function( link, min, max)
		local iLevel = GetItemValue( 'level', link)
		local iMin = tonumber( min) or 0
		local iMax = tonumber( max)
		if iMax then
			return iMax >= iLevel and iLevel >= iMin
		else
			return iLevel >= iMin
		end
	end,
	
	['uselevel'] = function( link, min)
		local iLevel = GetItemValue( 'uselevel', link)
		local iMin = tonumber( min) or 0
		return iLevel >= iMin
	end,
	
	['type'] = function( link, ...)
		local iType = GetItemValue( 'type', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			if iType == value then
				return true
			end
		end
		return false
	end,
	
	['subtype'] = function( link, ...)
		local iSubType = GetItemValue( 'subtype', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			if iSubType == value then
				return true
			end
		end
		return false
	end,
	
	['count'] = function( link, min)
		local iCount = GetItemValue( 'count', link)
		local iMin = tonumber( min) or 1
		return iCount >= iMin
	end,
	
	['equip'] = function( link, ...)
		local iEquip = GetItemValue( 'equip', link)
		for n = 1,select( '#', ...) do
			local value = select( n, ...)
			if iEquip == value then
				return true
			end
		end
		return false
	end,

	['useable'] = function( link)
		local iEquip = GetItemValue( 'equip', link)
		return iEquip and true or false
	end,

	['price'] = function( link, min, max)
		local iPrice = GetItemValue( 'price', link)
		local iMin = tonumber( min) or 0
		local iMax = tonumber( max)
		if iMax then
			return iMax >= iPrice and iPrice >= iMin
		else
			return iLevel >= iMin
		end
	end,
	
	['bag'] = function( link, ...)
		for n = 1,select( '#', ...) do
			local bagID = tonumber( select( n, ...))
			if bagID and 0 <= bagID and bagID <= NUM_BAG_SLOTS then
				for slotID = 1,GetContainerNumSlots( bagID) do
					local itemLink = GetContainerItemLink( bagID, slotID)
					if link == itemLink then
						return true
					end
				end
			end
		end
		return false
	end,

	['outfit'] = function( link, ...)
		if IsAddOnLoaded( 'Outfitter') and Outfitter and Outfitter:IsInitialized() then
			local info = Outfitter:GetItemInfoFromLink( link )
			local outfits = info and Outfitter:GetOutfitsUsingItem( info )
			if outfits and #outfits > 0 then
				for n = 1,select( '#', ...) do
					local value = select( n, ...)
					for _,outfit in pairs( outfits) do
						if outfit and outfit == value then
							return true
						end
					end
				end
			end
		end
		return false
	end,
	
	['periodictable'] = function( link, ...)
		local PT3 = LibStub( 'LibPeriodicTable-3.1', true)
		if PT3 then
			for n = 1,select( '#', ...) do
				local value = select( n, ...)
				if PT3:ItemInSet( link, value) then
					return true
				end
			end
		end
		return false
	end,
	
	['tooltip'] = function( link, ...)
		local GRA = LibStub( 'LibGratuity-3.0', true)
		if GRA then
			GRA:SetHyperlink( link)
			for n = 1,select( '#', ...) do
				local value = select( n, ...)
				if GRA:Find( value) then
					return true
				end
			end
		end
		return false
	end,
}
--[[---------------------------------------------------------------------------------
  lib functions
------------------------------------------------------------------------------------]]
local function Link( link)
	local id = GetItemID( link)
	if id then
		return 'ref@'..id
	end
	return link
end

local function Trim( s)
	return '"'..s:match( '^[%s"\']*(.-)[%s"\']*$')..'"'
end

local function Function( fkt, params)
	fkt = fkt:lower()
	params = params:gsub( '|c%x+|H[^%]]+%]|h|r', Link) -- '|c%s|Hitem:%i:0:0:0:0:0:0:0|h[%s]|h|r'
	params = params:gsub( 'ref@%d+', function( ref)
		local id = tonumber( ref:sub( 5))
		if id then
			return GetItemValue( fkt, id)
		end
		return ref
	end)
	return fkt..'(link,'..params:gsub( '[^,]+', Trim)
end

function lib:Compile( template)
	local code = tostring( template)
	code = code:gsub( '%s+', ' ')
	code = code:gsub( '(%a+)%(([^%)]*)', Function)
	return 'local link = ... return ' .. code
end

function lib:Execute( compiled, link)
	local fkt = GetCache( compiled)
	if not fkt then
		fkt = assert( loadstring( compiled))
		setfenv( fkt, Environment)
		SetCache( compiled, fkt)
	end
	return fkt( link)
end

function lib:IsValid( template, link)
	local compiled = self:Compile( template)
	return self:Execute( compiled, link)
end
